==> roccat-tools-1.0.0.tar.bz2: roccat-tools-1.0.0/koneplus/libkonepluseventhandler/koneplus_eventhandler_channel.c <==
/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "koneplus_eventhandler_channel.h"
#include "g_cclosure_roccat_marshaller.h"
#include "roccat/koneplus.h"
#include <sys/ioctl.h>

#define KONEPLUS_EVENTHANDLER_CHANNEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), KONEPLUS_EVENTHANDLER_CHANNEL_TYPE, KoneplusEventhandlerChannelClass))
#define IS_KONEPLUS_EVENTHANDLER_CHANNEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), KONEPLUS_EVENTHANDLER_CHANNEL_TYPE))
#define KONEPLUS_EVENTHANDLER_CHANNEL_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), KONEPLUS_EVENTHANDLER_CHANNEL_TYPE, KoneplusEventhandlerChannelPrivate))

typedef struct _KoneplusEventhandlerChannelClass KoneplusEventhandlerChannelClass;
typedef struct _KoneplusEventhandlerChannelPrivate KoneplusEventhandlerChannelPrivate;

struct _KoneplusEventhandlerChannel {
	GObject parent;
	KoneplusEventhandlerChannelPrivate *priv;
};

struct _KoneplusEventhandlerChannelClass {
	GObjectClass parent_class;
};

struct _KoneplusEventhandlerChannelPrivate {
	guint event_source_id;
	gint report_size;
	gboolean running;
};

enum {
	CPI_CHANGED, /* cpi_index */
	OPEN_DRIVER,
	PROFILE_CHANGED, /* profile_number */
	QUICKLAUNCH, /* key_index */
	SENSITIVITY_CHANGED, /* x, y */
	TALK, /* key_index, event */
	TIMER_START, /* key_index */
	TIMER_STOP,
	LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE(KoneplusEventhandlerChannel, koneplus_eventhandler_channel, G_TYPE_OBJECT);

static void process_chrdev_event(KoneplusEventhandlerChannel *eventhandler_channel, KoneplusChrdevEvent const *event) {
	switch (event->type) {
	case KONEPLUS_CHRDEV_EVENT_TYPE_PROFILE:
		g_signal_emit((gpointer)eventhandler_channel, signals[PROFILE_CHANGED], 0, event->data1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_CPI:
		g_signal_emit((gpointer)eventhandler_channel, signals[CPI_CHANGED], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_SENSITIVITY:
		g_signal_emit((gpointer)eventhandler_channel, signals[SENSITIVITY_CHANGED], 0, event->data1 - 6, event->data2 - 6);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_QUICKLAUNCH:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[QUICKLAUNCH], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_OPEN_DRIVER:
		g_signal_emit((gpointer)eventhandler_channel, signals[OPEN_DRIVER], 0);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TILT:
		/* just ignore this event */
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TIMER:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[TIMER_START], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TIMER_STOP:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[TIMER_STOP], 0);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_MULTIMEDIA:
		/* TODO implement multimedia handler */
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TALK:
		g_signal_emit((gpointer)eventhandler_channel, signals[TALK], 0, event->data1 - 1, event->data2);
		break;
	default:
		g_debug("unknown event = %02x%02x%02x", event->type, event->data1, event->data2);
	}
}

static gboolean io_cb(GIOChannel *channel, GIOCondition condition, gpointer user_data) {
	KoneplusEventhandlerChannel *eventhandler_channel = KONEPLUS_EVENTHANDLER_CHANNEL(user_data);
	KoneplusEventhandlerChannelPrivate *priv = eventhandler_channel->priv;
	gchar *event;
	GIOStatus status;
	GError *error = NULL;
	gsize length;

	if (condition & G_IO_HUP)
		return FALSE;

	event = g_malloc(priv->report_size);

	status = g_io_channel_read_chars(channel, event, priv->report_size, &length, &error);

	if (status == G_IO_STATUS_AGAIN)
		goto ok;

	if (status == G_IO_STATUS_ERROR || status == G_IO_STATUS_EOF) {
		g_critical("error reading io_channel");
		goto error;
	}

	if (length != priv->report_size) {
		g_critical("wrong length %zd %s", length, error->message);
		goto error;
	}

	switch (priv->report_size) {
	case sizeof(KoneplusChrdevEvent):
		process_chrdev_event(eventhandler_channel, (KoneplusChrdevEvent const *)event);
		break;
	default:
		g_critical("wrong event size");
		goto error;
	}

ok:
	g_free(event);
	return TRUE; // keep on going
error:
	g_free(event);
	return FALSE;
}

KoneplusEventhandlerChannel *koneplus_eventhandler_channel_new(void) {
	return KONEPLUS_EVENTHANDLER_CHANNEL(g_object_new(KONEPLUS_EVENTHANDLER_CHANNEL_TYPE, NULL));
}

static void koneplus_eventhandler_channel_init(KoneplusEventhandlerChannel *channel) {
	KoneplusEventhandlerChannelPrivate *priv = KONEPLUS_EVENTHANDLER_CHANNEL_GET_PRIVATE(channel);
	channel->priv = priv;
	priv->running = FALSE;
}

gboolean koneplus_eventhandler_channel_start(KoneplusEventhandlerChannel *eventhandler_channel, gchar const *path, GError **error) {
	KoneplusEventhandlerChannelPrivate *priv = eventhandler_channel->priv;
	GIOChannel *channel;
	GIOStatus status;
	int fd;
	
	channel = g_io_channel_new_file(path, "r", error);
	if (!channel)
		return FALSE;

	status = g_io_channel_set_encoding(channel, NULL, error);
	if (status != G_IO_STATUS_NORMAL) {
		g_io_channel_unref(channel);
		return FALSE;
	}

	fd = g_io_channel_unix_get_fd(channel);
	if (ioctl(fd, ROCCATIOCGREPSIZE, &priv->report_size) == -1)
		priv->report_size = sizeof(KoneplusChrdevEvent);

	if (priv->report_size != sizeof(KoneplusChrdevEvent)) {
		g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_FAILED, "ioctl reported unsupported event size %i", priv->report_size);
		g_io_channel_unref(channel);
		return FALSE;
	}

	priv->event_source_id = g_io_add_watch(channel, G_IO_IN | G_IO_HUP, (GIOFunc)io_cb, eventhandler_channel);
	/* g_io_add_watch increases reference count
	 * this passes ownership of the channel to the watch */
	g_io_channel_unref(channel);

	return TRUE;
}

void koneplus_eventhandler_channel_stop(KoneplusEventhandlerChannel *eventhandler_channel) {
	KoneplusEventhandlerChannelPrivate *priv = eventhandler_channel->priv;
	if (priv->running) {
		g_source_remove(priv->event_source_id);
		priv->running = FALSE;
	}
}

static void finalize(GObject *object) {
	KoneplusEventhandlerChannel *eventhandler_channel = KONEPLUS_EVENTHANDLER_CHANNEL(object);
	koneplus_eventhandler_channel_stop(eventhandler_channel);
	G_OBJECT_CLASS(koneplus_eventhandler_channel_parent_class)->finalize(object);
}

static void koneplus_eventhandler_channel_class_init(KoneplusEventhandlerChannelClass *klass) {
	GObjectClass *gobject_class;

	gobject_class = (GObjectClass *)klass;

	gobject_class->finalize = finalize;

	g_type_class_add_private(klass, sizeof(KoneplusEventhandlerChannelPrivate));
	
	signals[CPI_CHANGED] = g_signal_new("cpi-changed", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[OPEN_DRIVER] = g_signal_new("open-driver", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
			0);

	signals[PROFILE_CHANGED] = g_signal_new("profile-changed", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[QUICKLAUNCH] = g_signal_new("quicklaunch", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[SENSITIVITY_CHANGED] = g_signal_new("sensitivity-changed", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_roccat_marshal_VOID__UCHAR_UCHAR, G_TYPE_NONE,
			2, G_TYPE_UCHAR, G_TYPE_UCHAR);

	signals[TALK] = g_signal_new("talk", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_roccat_marshal_VOID__UCHAR_UCHAR, G_TYPE_NONE,
			2, G_TYPE_UCHAR, G_TYPE_UCHAR);

	signals[TIMER_START] = g_signal_new("timer-start", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[TIMER_STOP] = g_signal_new("timer-stop", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
			0);
}

==> roccat-tools-1.6.0.tar.bz2: roccat-tools-1.6.0/koneplus/libkonepluseventhandler/koneplus_eventhandler_channel.c <==
/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "koneplus_eventhandler_channel.h"
#include "g_cclosure_roccat_marshaller.h"
#include "koneplus.h"
#include "roccat.h"
#include <sys/ioctl.h>

#define KONEPLUS_EVENTHANDLER_CHANNEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), KONEPLUS_EVENTHANDLER_CHANNEL_TYPE, KoneplusEventhandlerChannelClass))
#define IS_KONEPLUS_EVENTHANDLER_CHANNEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), KONEPLUS_EVENTHANDLER_CHANNEL_TYPE))
#define KONEPLUS_EVENTHANDLER_CHANNEL_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), KONEPLUS_EVENTHANDLER_CHANNEL_TYPE, KoneplusEventhandlerChannelPrivate))

typedef struct _KoneplusEventhandlerChannelClass KoneplusEventhandlerChannelClass;
typedef struct _KoneplusEventhandlerChannelPrivate KoneplusEventhandlerChannelPrivate;

struct _KoneplusEventhandlerChannel {
	GObject parent;
	KoneplusEventhandlerChannelPrivate *priv;
};

struct _KoneplusEventhandlerChannelClass {
	GObjectClass parent_class;
};

struct _KoneplusEventhandlerChannelPrivate {
	guint event_source_id;
	gint report_size;
	gboolean running;
};

enum {
	CPI_CHANGED, /* cpi_index */
	OPEN_DRIVER,
	PROFILE_CHANGED, /* profile_number */
	QUICKLAUNCH, /* key_index */
	SENSITIVITY_CHANGED, /* x, y */
	TALK, /* key_index, event */
	TIMER_START, /* key_index */
	TIMER_STOP,
	LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE(KoneplusEventhandlerChannel, koneplus_eventhandler_channel, G_TYPE_OBJECT);

static void process_chrdev_event(KoneplusEventhandlerChannel *eventhandler_channel, KoneplusChrdevEvent const *event) {
	switch (event->type) {
	case KONEPLUS_CHRDEV_EVENT_TYPE_PROFILE:
		g_signal_emit((gpointer)eventhandler_channel, signals[PROFILE_CHANGED], 0, event->data1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_CPI:
		g_signal_emit((gpointer)eventhandler_channel, signals[CPI_CHANGED], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_SENSITIVITY:
		g_signal_emit((gpointer)eventhandler_channel, signals[SENSITIVITY_CHANGED], 0, event->data1, event->data2);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_QUICKLAUNCH:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[QUICKLAUNCH], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_OPEN_DRIVER:
		g_signal_emit((gpointer)eventhandler_channel, signals[OPEN_DRIVER], 0);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TILT:
		/* just ignore this event */
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TIMER:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[TIMER_START], 0, event->data1 - 1);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TIMER_STOP:
		if (event->data2 == KONEPLUS_CHRDEV_EVENT_ACTION_PRESS)
			g_signal_emit((gpointer)eventhandler_channel, signals[TIMER_STOP], 0);
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_MULTIMEDIA:
		/* TODO implement multimedia handler */
		break;
	case KONEPLUS_CHRDEV_EVENT_TYPE_TALK:
		g_signal_emit((gpointer)eventhandler_channel, signals[TALK], 0, event->data1 - 1, event->data2);
		break;
	default:
		g_debug("unknown event = %02x%02x%02x", event->type, event->data1, event->data2);
	}
}

static gboolean io_cb(GIOChannel *channel, GIOCondition condition, gpointer user_data) {
	KoneplusEventhandlerChannel *eventhandler_channel = KONEPLUS_EVENTHANDLER_CHANNEL(user_data);
	KoneplusEventhandlerChannelPrivate *priv = eventhandler_channel->priv;
	gchar *event;
	GIOStatus status;
	GError *error = NULL;
	gsize length;

	if (condition & G_IO_HUP)
		return FALSE;

	event = g_malloc(priv->report_size);

	status = g_io_channel_read_chars(channel, event, priv->report_size, &length, &error);

	if (status == G_IO_STATUS_AGAIN)
		goto ok;

	if (status == G_IO_STATUS_ERROR || status == G_IO_STATUS_EOF) {
		g_critical("error reading io_channel");
		goto error;
	}

	if (length != priv->report_size) {
		g_critical("wrong length %zd %s", length, error->message);
		goto error;
	}

	switch (priv->report_size) {
	case sizeof(KoneplusChrdevEvent):
		process_chrdev_event(eventhandler_channel, (KoneplusChrdevEvent const *)event);
		break;
	default:
		g_critical("wrong event size");
		goto error;
	}

ok:
	g_free(event);
	return TRUE; // keep on going
error:
	g_free(event);
	return FALSE;
}

KoneplusEventhandlerChannel *koneplus_eventhandler_channel_new(void) {
	return KONEPLUS_EVENTHANDLER_CHANNEL(g_object_new(KONEPLUS_EVENTHANDLER_CHANNEL_TYPE, NULL));
}

static void koneplus_eventhandler_channel_init(KoneplusEventhandlerChannel *channel) {
	KoneplusEventhandlerChannelPrivate *priv = KONEPLUS_EVENTHANDLER_CHANNEL_GET_PRIVATE(channel);
	channel->priv = priv;
	priv->running = FALSE;
}

gboolean koneplus_eventhandler_channel_start(KoneplusEventhandlerChannel *eventhandler_channel, gchar const *path, GError **error) {
	KoneplusEventhandlerChannelPrivate *priv = eventhandler_channel->priv;
	GIOChannel *channel;
	GIOStatus status;
	int fd;
	
	channel = g_io_channel_new_file(path, "r", error);
	if (!channel)
		return FALSE;

	status = g_io_channel_set_encoding(channel, NULL, error);
	if (status != G_IO_STATUS_NORMAL) {
		g_io_channel_unref(channel);
		return FALSE;
	}

	fd = g_io_channel_unix_get_fd(channel);
	if (ioctl(fd, ROCCATIOCGREPSIZE, &priv->report_size) == -1)
		priv->report_size = sizeof(KoneplusChrdevEvent);

	if (priv->report_size != sizeof(KoneplusChrdevEvent)) {
		g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_FAILED, "ioctl reported unsupported event size %i", priv->report_size);
		g_io_channel_unref(channel);
		return FALSE;
	}

	priv->event_source_id = g_io_add_watch(channel, G_IO_IN | G_IO_HUP, (GIOFunc)io_cb, eventhandler_channel);
	/* g_io_add_watch increases reference count
	 * this passes ownership of the channel to the watch */
	g_io_channel_unref(channel);

	return TRUE;
}

void koneplus_eventhandler_channel_stop(KoneplusEventhandlerChannel *eventhandler_channel) {
	KoneplusEventhandlerChannelPrivate *priv = eventhandler_channel->priv;
	if (priv->running) {
		g_source_remove(priv->event_source_id);
		priv->running = FALSE;
	}
}

static void finalize(GObject *object) {
	KoneplusEventhandlerChannel *eventhandler_channel = KONEPLUS_EVENTHANDLER_CHANNEL(object);
	koneplus_eventhandler_channel_stop(eventhandler_channel);
	G_OBJECT_CLASS(koneplus_eventhandler_channel_parent_class)->finalize(object);
}

static void koneplus_eventhandler_channel_class_init(KoneplusEventhandlerChannelClass *klass) {
	GObjectClass *gobject_class;

	gobject_class = (GObjectClass *)klass;

	gobject_class->finalize = finalize;

	g_type_class_add_private(klass, sizeof(KoneplusEventhandlerChannelPrivate));
	
	signals[CPI_CHANGED] = g_signal_new("cpi-changed", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[OPEN_DRIVER] = g_signal_new("open-driver", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
			0);

	signals[PROFILE_CHANGED] = g_signal_new("profile-changed", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[QUICKLAUNCH] = g_signal_new("quicklaunch", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[SENSITIVITY_CHANGED] = g_signal_new("sensitivity-changed", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_roccat_marshal_VOID__UCHAR_UCHAR, G_TYPE_NONE,
			2, G_TYPE_UCHAR, G_TYPE_UCHAR);

	signals[TALK] = g_signal_new("talk", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_roccat_marshal_VOID__UCHAR_UCHAR, G_TYPE_NONE,
			2, G_TYPE_UCHAR, G_TYPE_UCHAR);

	signals[TIMER_START] = g_signal_new("timer-start", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__UCHAR, G_TYPE_NONE,
			1, G_TYPE_UCHAR);

	signals[TIMER_STOP] = g_signal_new("timer-stop", KONEPLUS_EVENTHANDLER_CHANNEL_TYPE,
			G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
			0);
}
